// -*- C++ -*-
/*!
 * @file PortCallBack.h
 * @brief PortCallBack class
 * @date $Date: 2007-12-31 03:08:05 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PortCallBack.h 1225 2009-02-28 02:30:25Z n-ando $
 *
 */

#ifndef PortCallBack_h
#define PortCallBack_h

namespace RTC
{
  //============================================================
  // callback functor base classes
  /*!
   * @if jp
   * @class OnWrite
   * @brief write() ΥХåݥ饹
   *
   * DataPortΥХåե˥ǡwrite()ľ˸ƤӽФ륳Хå
   * 󥿡ե
   *
   * @param DataType Хåե˽񤭹ǡ
   *
   * @since 0.4.0
   *
   * @else
   * @class OnWrite
   * @brief Callback abstract class on write()
   *
   * This is the interface for callback invoked immediately before
   * data is done write() into the DataPort's buffer.
   *
   * @param DataType Data type to write into the buffer
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  struct OnWrite
  {
    /*!
     * @if jp
     *
     * @brief ǥȥ饯
     *
     * ǥȥ饯
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OnWrite(void){}
    
    /*!
     * @if jp
     *
     * @brief Хå᥽å
     *
     * Хåե˥ǡ񤭹ޤľ˸ƤӽФ륳Хå᥽å
     *
     * @param value Хåե˽񤭹ޤǡ
     *
     * @else
     *
     * @brief Callback method
     *
     * This is the callback method invoked immediately before data is written
     * into the buffer.
     *
     * @param value Data that is written into the buffer
     *
     * @endif
     */
    virtual void operator()(const DataType& value) = 0;
  };
  
  /*!
   * @if jp
   * @class OnWriteConvert
   * @brief write() ΥǡѴХåݥ饹
   *
   * InPort/OutPortΥХåե˥ǡ write()˸ƤӽФ
   * Хåѥ󥿡ե
   * ΥХåͤХåե˳Ǽ롣
   *
   * @since 0.4.0
   *
   * @else
   * @class OnWriteConvert
   * @brief Data convert callback abstract class on write()
   *
   * This is the interface for callback invoked when data is done write()
   * into the InPort/OutPort's buffer.
   * The return value of this callback will be stored in the buffer.
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  struct OnWriteConvert
  {
    /*!
     * @if jp
     *
     * @brief ǥȥ饯
     *
     * ǥȥ饯
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OnWriteConvert(void){}
    
    /*!
     * @if jp
     *
     * @brief Хå᥽å
     *
     * Хåե˥ǡ񤭹ޤݤ˸ƤӽФ륳Хå᥽å
     *
     * @param value Ѵǡ
     *
     * @return Ѵǡ
     *
     * @else
     *
     * @brief Callback method
     *
     * This is the callback method invoked when data is written into the
     * buffer.
     *
     * @param value Data before the convert
     *
     * @return Data after the convert
     *
     * @endif
     */
    virtual DataType operator()(const DataType& value) = 0;
  };
  
  /*!
   * @if jp
   * @class OnRead
   * @brief read() ΥХåݥ饹
   *
   * InPort/OutPortΥХåեǡ read()ľ˸ƤӽФ
   * Хåѥ󥿡ե
   *
   * @since 0.4.0
   *
   * @else
   * @class OnRead
   * @brief Callback abstract class on read()
   *
   * This is the interface for callback invoked immediately before
   * data is done read() from the InPort/OutPort's buffer.
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  struct OnRead
  {
    /*!
     * @if jp
     *
     * @brief ǥȥ饯
     *
     * ǥȥ饯
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OnRead(void){}
    
    /*!
     * @if jp
     *
     * @brief Хå᥽å
     *
     * Хåեǡɤ߽Фľ˸ƤӽФ륳Хå᥽å
     *
     * @else
     *
     * @brief Callback method
     *
     * This is the callback method invoked immediately before data is readout
     * from the buffer.
     *
     * @endif
     */
    virtual void operator()() = 0;
  };
  
  /*!
   * @if jp
   * @class OnReadConvert
   * @brief read() ΥǡѴХåݥ饹
   *
   * InPort/OutPortΥХåեǡ read()ݤ˸ƤӽФ
   * Хåѥ󥿡ե
   * ΥХåͤread()ͤȤʤ롣
   *
   * @since 0.4.0
   *
   * @else
   * @class OnReadConvert
   * @brief Data convert callback abstract class on read()
   *
   * This is the interface for callback invoked when data is done read()
   * from the InPort/OutPort's buffer.
   * The return value of this callback will be the return value of read().
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  struct OnReadConvert
  {
    /*!
     * @if jp
     *
     * @brief ǥȥ饯
     *
     * ǥȥ饯
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OnReadConvert(void){}
    
    /*!
     * @if jp
     *
     * @brief Хå᥽å
     *
     * Хåեǡɤ߽Фݤ˸ƤӽФ륳Хå᥽å
     *
     * @else
     *
     * @brief Callback method
     *
     * This is the callback method invoked when data is readout from the
     * buffer.
     *
     * @endif
     */
    virtual DataType operator()(const DataType& value) = 0;
  };
  
  /*!
   * @if jp
   * @class OnOverflow
   * @brief ХåեСեΥХåݥ饹
   *
   * InPort/OutPortΥХåե˥ǡput()ХåեСե
   * ˸ƤФ륳Хå᥽åɡ
   *
   * @since 0.4.0
   *
   * @else
   * @class OnOverflow
   * @brief Callback abstract class when the buffer overflow occurs
   *
   * This is the callback method invoked when data is done put()
   * into the InPort/OutPort's buffer or when the buffer overflow occurs.
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  struct OnOverflow
  {
    /*!
     * @if jp
     *
     * @brief ǥȥ饯
     *
     * ǥȥ饯
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OnOverflow(void){}
    
    /*!
     * @if jp
     *
     * @brief Хå᥽å
     *
     * ХåեСե˸ƤӽФ륳Хå᥽å
     *
     * @param value Хåեؽ񤭹ǡ
     *
     * @else
     *
     * @brief Callback method
     *
     * This is the callback method invoked when the buffer overflow occurs.
     *
     * @param value Data that is written into the buffer
     *
     * @endif
     */
    virtual void operator()(const DataType& value) = 0;
  };
  
  /*!
   * @if jp
   * @class OnUnderflow
   * @brief Underflow ΥХåݥ饹
   *
   * @since 0.4.0
   *
   * InPort/OutPortΥХåե˥ǡread()ˡɤ߽Ф٤ǡ
   * ʤ˸ƤӽФ륳Хå󥿥ե
   * ΥХåͤread()ͤȤʤ롣
   *
   * @else
   * @class OnUnderflow
   * @brief Callback abstract class on underflow
   *
   * @since 0.4.0
   *
   * This is the interface for callback invoked when data is done read()
   * into the InPort/OutPort's buffer and the readout data does not exist.
   * The return value of this callback will be the return value of read().
   *
   * @endif
   */
  template <class DataType>
  struct OnUnderflow
  {
    /*!
     * @if jp
     *
     * @brief ǥȥ饯
     *
     * ǥȥ饯
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OnUnderflow(void){}
    
    /*!
     * @if jp
     *
     * @brief Хå᥽å
     *
     * Хåեե˸ƤӽФ륳Хå᥽å
     *
     * @return ؤȤʤɤ߽Фǡ
     *
     * @else
     *
     * @brief Callback method
     *
     * This is the callback method invoked when the buffer underflow occurs.
     *
     * @return Substituted readout data
     *
     * @endif
     */
    virtual DataType operator()() = 0;
  };
  
  /*!
   * @if jp
   * @class OnWriteTimeout
   * @brief ॢȻΥХåݥ饹
   *
   * InPort/OutPortΥХåե˥ǡwrite()ݤˡॢȤȯ
   * ˸ƤӽФ륳Хå󥿥ե
   *
   * @since 0.4.0
   *
   * @else
   * @class OnWriteTimeout
   * @brief Callback abstract class on timeout
   *
   * This is the interface for callback invoked when data is done write()
   * into the InPort/OutPort's buffer and the timeout occurred.
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  struct OnWriteTimeout
  {
    /*!
     * @if jp
     *
     * @brief ǥȥ饯
     *
     * ǥȥ饯
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OnWriteTimeout(void){}
    
    /*!
     * @if jp
     *
     * @brief Хå᥽å
     *
     * ॢȯ˸ƤӽФ륳Хå᥽å
     *
     * @param value Хåեؽ񤭹ǡ
     *
     * @else
     *
     * @brief Callback method
     *
     * This is the callback method invoked when the timeout occurs.
     *
     * @param value Data that is written into the buffer
     *
     * @endif
     */
    virtual void operator()(const DataType& value) = 0;
  };
  
  /*!
   * @if jp
   * @class OnReadTimeout
   * @brief ॢȻΥХåݥ饹
   *
   * InPort/OutPortΥХåե˥ǡread()ݤˡॢȤȯ
   * ˸ƤӽФ륳Хå󥿥ե
   *
   * @since 0.4.0
   *
   * @else
   * @class OnReadTimeout
   * @brief OnReadTimeout abstract class
   *
   * This is the interface for callback invoked when data is done read()
   * into the InPort/OutPort's buffer and the timeout occurred.
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  struct OnReadTimeout
  {
    /*!
     * @if jp
     *
     * @brief ǥȥ饯
     *
     * ǥȥ饯
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OnReadTimeout(void){}
    
    /*!
     * @if jp
     *
     * @brief Хå᥽å
     *
     * ॢȯ˸ƤӽФ륳Хå᥽å
     *
     * @return ؤȤʤɤ߽Фǡ
     *
     * @else
     *
     * @brief Callback method
     *
     * This is the callback method invoked when the timeout occurs.
     *
     * @return Substituted readout data
     *
     * @endif
     */
    virtual DataType operator()() = 0;
  };

  class PublisherBase;
  class OnConnect
  {
  public:
    virtual ~OnConnect(void){}
    virtual void operator()(const char* id, PublisherBase* publisher) = 0;
  };

  class OnDisconnect
  {
  public:
    virtual ~OnDisconnect(void){}
    virtual void operator()(const char* id) = 0;
  };


};
#endif // PortCallBack_h
