// -*- C++ -*-
/*!
 * @file   RingBufferTests.cpp
 * @brief  RingBuffer test class
 * @date   $Date: 2008/01/24 01:52:14 $
 * @author Shinji Kurihara
 *         Noriaki Ando <n-ando@aist.go.jp>
 * 
 * Copyright (C) 2006
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: RingBufferTests.cpp 1341 2009-05-18 06:03:07Z n-ando $
 *
 */

/*
 * $Log: RingBufferTests.cpp,v $
 * Revision 1.3  2008/01/24 01:52:14  tsakamoto
 * *** empty log message ***
 *
 * Revision 1.2  2008/01/11 11:27:11  arafune
 * *** empty log message ***
 *
 * Revision 1.1  2007/12/20 07:50:17  arafune
 * *** empty log message ***
 *
 * Revision 1.2  2006/12/02 18:53:08  n-ando
 * Some tests were added.
 *
 * Revision 1.1  2006/11/27 08:37:03  n-ando
 * TestSuites are devided into each directory.
 *
 *
 */

#ifndef RingBuffer_cpp
#define RingBuffer_cpp

#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/TextOutputter.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/TestAssert.h>

#include <string>
#include <sstream>
#include <rtm/RingBuffer.h>

//#define DEBUG

namespace RingBuffer
{
  /*!
   * @class RingBufferTests class
   * @brief RingBuffer test
   */
  class RingBufferTests
    : public CppUnit::TestFixture
  {
    CPPUNIT_TEST_SUITE(RingBufferTests);
		
//    CPPUNIT_TEST(test_length);
//    CPPUNIT_TEST(test_isFull);
//    CPPUNIT_TEST(test_isEmpty);
//    CPPUNIT_TEST(test_init);
    CPPUNIT_TEST(test_write_read);
//    CPPUNIT_TEST(test_write_read_with_small_length);
//    CPPUNIT_TEST(test_isNew);
		
    CPPUNIT_TEST_SUITE_END();
		
  private:
		
  public:
    /*!
     * @brief Constructor
     */
    RingBufferTests()
    {
    }
		
    /*!
     * @brief Destructor
     */
    ~RingBufferTests()
    {
    }
		
    /*!
     * @brief Test initialization
     */
    virtual void setUp()
    {
    }
		
    /*!
     * @brief Test finalization
     */
    virtual void tearDown()
    { 
    }
		
    /*!
     * @brief length()᥽åɤΥƥ
     * 
     * - 󥹥ȥ饯ǻꤵ줿ХåեĹǤ뤫
     */
    void test_length()
    {
      RTC::RingBuffer<int> buff(123);
      CPPUNIT_ASSERT(buff.length() == 123);
    }

    /*!
     * @brief isEmpty()᥽åɤΥƥ
     * 
     * - Хåեľ塢ǤϤʤȽꤵ뤫
     * - Ǹ˥ǡɤ߼줿塢ǡ񤭹ߤƤʤ硢
     *   Ƚꤵ뤫
     * - Ǹ˥ǡɤ߼줿塢ǡ񤭹ߤ줿硢
     *   ǤϤʤȽꤵ뤫
     */
    void test_isEmpty()
    {
      long int length = 10;
      RTC::RingBuffer<int> buff(length);
			
      // (1) Хåեľ塢ǤϤʤȽꤵ뤫
      int initialValue = 12345;
      for (unsigned int i(0); i < buff.length(); ++i)
        {
          buff.write(initialValue);
        }
      CPPUNIT_ASSERT_EQUAL(false, buff.empty());			
			
      // (2) Ǹ˥ǡɤ߼줿塢ǡ񤭹ߤ
      // ʤ硢Ƚꤵ뤫
      int readValue;
      CPPUNIT_ASSERT_EQUAL(RTC::BufferBase<int>::BUFFER_OK, buff.read(readValue));
      CPPUNIT_ASSERT_EQUAL(true, buff.empty());
			
      // (3) Ǹ˥ǡɤ߼줿塢ǡ񤭹ߤ줿
      // 硢ǤϤʤȽꤵ뤫
      int writeValue = 98765;
      CPPUNIT_ASSERT_EQUAL(RTC::BufferBase<int>::BUFFER_OK, buff.write(writeValue));
      CPPUNIT_ASSERT_EQUAL(false, buff.empty());
    }
		
    /*!
     * @brief isEmpty()᥽åɤΥƥ
     * @attention ܥƥȤϡRingBufferμͤ󥰾Хåեб
     *            줿ΤФƥƤˤʤäƤ롣
     *            󥰾ХåեбRinguBufferǤܥƥȤϼԤ롣
     *
     * - ǸΣǡĤɤ߼ꡢȽꤵʤȤǧ
     * - ǸΣǡޤɤ߼ꡢȽꤵ뤳Ȥǧ
     */
    void _test_isEmpty()
    {
      // Хåե
      long int length = 10;
      RTC::RingBuffer<int> buff(length);
			
      int value = 12345;
      for (unsigned int i(0); i < buff.length(); ++i)
        {
          buff.write(value);
        }

      // (1) ǸΣǡĤɤ߼ꡢȽꤵʤȤǧ
      for (long int i = 0; i < length - 1; i++) {
	int data;
	buff.read(data);
	CPPUNIT_ASSERT_EQUAL(false, buff.empty());
      }

      // (2) ǸΣǡޤɤ߼ꡢȽꤵ뤳Ȥǧ
      int data;
      buff.read(data);
      CPPUNIT_ASSERT_EQUAL(true, buff.empty());
    }
		
    /*!
     * @brief isFull()᥽åɤΥƥ
     * 
     * - Хåեξ硢եȽϵȤʤ뤫
     * - Хåե˥ǡ񤭹ޤƤ֤Ǥ⡢եȽϵȤʤ뤫
     * - Хåե˴ʬǡ񤭹ޤƤ֤ǡեȽϵȤʤ뤫
     */
    void test_isFull()
    {
      // (1) Хåեξ硢եȽϵȤʤ뤫
      int length1 = 10;
      RTC::RingBuffer<int> buff1(length1);
      CPPUNIT_ASSERT_EQUAL(false, buff1.full());
			
      // (2) Хåե˥ǡ񤭹ޤƤ֤Ǥ⡢եȽϵ
      // Ȥʤ뤫
      int length2 = 10;
      RTC::RingBuffer<int> buff2(length2);
      for (int i = 0; i < length2; i++) {
	buff2.write(i);
      }
      CPPUNIT_ASSERT_EQUAL(false, buff2.full());
			
      // (3) Хåե˴ʬǡ񤭹ޤƤ֤ǡեȽϵ
      // Ȥʤ뤫
      int length3 = 10;
      RTC::RingBuffer<int> buff3(length3);
      for (int i = 0; i < length3 / 2; i++) {
	buff3.write(i);
      }
      CPPUNIT_ASSERT_EQUAL(false, buff3.full());
    }
				
    /*!
     * @brief init()᥽åɤΥƥ
     * 
     * - 餫ǡǽ塢ꤷǡɤ߽Ф뤫
     */
    void test_init()
    {
      // Хåեơinit()ǽ
      long int length = 10;
      RTC::RingBuffer<int> buff(length);
			
      int value = 12345;
      for (unsigned int i(0); i < buff.length(); ++i)
        {
          buff.write(value);
        }
			
      // ꤷǡɤ߽Ф뤫
      int expected = 12345;
      for (long int i = 0; i < length; i++) {
	int actual;
	buff.read(actual);
	CPPUNIT_ASSERT_EQUAL(expected, actual);
      }
    }

    /*!
     * @brief write()᥽åɤread()᥽åɤΥƥ
     * 
     * - Хåե֤ǣǡɽФԤ񤭹ǡ
     *   ɤ߽Ф뤫
     * - Хåե˥ǡ񤭹ޤƤ֤ǣǡɽФԤ
     *   񤭹ǡɤ߽Ф뤫
     * - Хåե˴ʬǡ񤭹ޤƤ֤ǣǡɽФ
     *   Ԥ񤭹ǡɤ߽Ф뤫
     */
    void test_write_read()
    {
      // (1) Хåե֤ǣǡɽФԤ񤭹ǡ
      // ɤ߽Ф뤫ХåեΤޤޤˤ
      long int length1 = 3;
      RTC::RingBuffer<int> buff1(length1);
      coil::Properties prop;
      prop["write.full_policy"] = "block";
      prop["write.timeout"] = "5.0";
      prop["read.empty_policy"] = "block";
      prop["read.timeout"] = "5.0";

      buff1.init(prop);
      // ǡɽФԤ
      for (int writeValue = 0; writeValue < 100; writeValue++) {
	// 
        if (buff1.full())
          {
            std::cout << "### FULL ###" << std::endl;
            int readValue;

            if (writeValue % 5 == 0)
              {
                while (!buff1.empty())
                  {
                    std::cout << "read timeout: " << 5 << std::endl;
                    buff1.read(readValue, 5);
                    std::cout << "read: " << readValue << std::endl;
                  }
                std::cout << "read timeout: " << 5 << std::endl;
                std::cout << "waiting 5 sec" << std::endl;
                std::cout << "read ret: " << RTC::BufferStatus::toString(buff1.read(readValue, 5)) << std::endl;
                std::cout << "read: " << readValue << std::endl;
              }
            else
              {
                buff1.read(readValue);
                std::cout << "read: " << readValue << std::endl;
              }

            if (buff1.full())
              {
                std::cout << "??? still full" << std::endl;
              }
            else
              {
                std::cout << "buffer full was blown over." << std::endl;
              }
          }
        if (buff1.empty())
          {
            std::cout << "### EMPTY ###" << std::endl;
          }

        //        std::cout << "timeout-> " << writeValue << std::endl;
        std::cout << "write ret: " << RTC::BufferStatus::toString(buff1.write(writeValue, writeValue)) << std::endl;
				
        //        std::cout << "readable " << buff1.readable() << std::endl;
        //        std::cout << "writable " << buff1.writable() << std::endl;

	// ɽФ
	int readValue;
	buff1.get(readValue);
			
        std::cout << writeValue << " == " << readValue << std::endl;

        //        buff1.read(readValue)
	// 񤭹ǡɤ߽Ф
        //	CPPUNIT_ASSERT_EQUAL(writeValue, readValue);
        sleep(1);
      }
      return;
      // (2) Хåե˥ǡ񤭹ޤƤ֤ǣǡ
      // ФԤ񤭹ǡɤ߽Ф뤫Хåե
      // ե֤ˤ
      long int length2 = 10;
      RTC::RingBuffer<int> buff2(length2);
      for (int i = 0; i < length2; i++) {
	buff2.write(i + 123);
      }
			
      // ǡɽФԤ
      for (int writeValue = 0; writeValue < 100; writeValue++) {
	// 
	buff2.write(writeValue);
				
	// ɽФ
	int readValue;
	buff2.read(readValue);
				
	// 񤭹ǡɤ߽Ф
	CPPUNIT_ASSERT_EQUAL(writeValue, readValue);
      }
			
      // (3) Хåե˴ʬǡ񤭹ޤƤ֤ǣǡ
      // ɽФԤ񤭹ǡɤ߽Ф뤫
      long int length3 = 10;
      RTC::RingBuffer<int> buff3(length3);
      for (int i = 0; i < length3 / 2; i++) {
	buff3.write(i + 123);
      }
			
      // ǡɽФԤ
      for (int writeValue = 0; writeValue < 100; writeValue++) {
	// 
	buff3.write(writeValue);
				
	// ɽФ
	int readValue;
	buff3.read(readValue);
				
	// 񤭹ǡɤ߽Ф
	CPPUNIT_ASSERT_EQUAL(writeValue, readValue);
      }
    }
		
    /*!
     * @brief write()᥽åɤread()᥽åɤΥƥȡʥХåեĹξ
     * 
     * - Хåե֤ǣǡɽФԤ񤭹ǡ
     *   ɤ߽Ф뤫
     * - Хåե˥ǡ񤭹ޤƤ֤ǣǡɽФԤ
     *   񤭹ǡɤ߽Ф뤫
     * - Хåե˴ʬǡ񤭹ޤƤ֤ǣǡɽФԤ
     *   񤭹ǡɤ߽Ф뤫
     */
    void test_write_read_with_small_length()
    {
      // (1) Хåե֤ǣǡɽФԤ񤭹ǡ
      // ɤ߽Ф뤫ХåեΤޤޤˤ
      long int length1 = 2;
      RTC::RingBuffer<int> buff1(length1);
			
      // ǡɽФԤ
      for (int writeValue = 0; writeValue < 100; writeValue++) {
	// 
	buff1.write(writeValue);
				
	// ɽФ
	int readValue;
	buff1.read(readValue);
				
	// 񤭹ǡɤ߽Ф
	CPPUNIT_ASSERT_EQUAL(writeValue, readValue);
      }
			
      // (2) Хåե˥ǡ񤭹ޤƤ֤ǣǡ
      // ФԤ񤭹ǡɤ߽Ф뤫Хåե
      // ե֤ˤ
      long int length2 = 2;
      RTC::RingBuffer<int> buff2(length2);
      for (int i = 0; i < length2; i++) {
	buff2.write(i + 123);
      }
			
      // ǡɽФԤ
      for (int writeValue = 0; writeValue < 100; writeValue++) {
	// 
	buff2.write(writeValue);
				
	// ɽФ
	int readValue;
	buff2.read(readValue);
				
	// 񤭹ǡɤ߽Ф
	CPPUNIT_ASSERT_EQUAL(writeValue, readValue);
      }
			
      // (3) Хåե˴ʬǡ񤭹ޤƤ֤ǣǡɽФԤ񤭹ǡɤ߽Ф뤫
      long int length3 = 2;
      RTC::RingBuffer<int> buff3(length3);
      for (int i = 0; i < 1; i++) {
	buff3.write(i + 123);
      }
			
      // ǡɽФԤ
      for (int writeValue = 0; writeValue < 100; writeValue++) {
	// 
	buff3.write(writeValue);
				
	// ɽФ
	int readValue;
	buff3.read(readValue);
				
	// 񤭹ǡɤ߽Ф
	CPPUNIT_ASSERT_EQUAL(writeValue, readValue);
      }
    }
		
    /*!
     * @brief isNew()᥽åɤΥƥ
     * 
     * - Хåեξ֤ǡisNewȽ꤬ˤʤ뤫
     * - Хåե˥ǡ񤭹ޤƤ֤ǡ
     *   ǡisNewȽ꤬ˤʤ뤫
     * - Хåե˥ǡ񤭹ޤƤ֤ǡǡ
     *   ΥǡɽФԤäisNewȽ꤬ˤʤ뤫
     * - Хåե˴ʬǡ񤭹ޤƤ֤ǡǡisNew
     *   Ƚ꤬ˤʤ뤫
     * - Хåե˴ʬǡ񤭹ޤƤ֤ǡǡ
     *   ΥǡɽФԤäisNewȽ꤬ˤʤ뤫
     */
    void test_isNew()
    {
      // (1) Хåեξ֤ǡisNewȽ꤬ˤʤ뤫
      long int length1 = 10;
      RTC::RingBuffer<int> buff1(length1);
      CPPUNIT_ASSERT_EQUAL(true, buff1.empty());
			
      // Хåե˥ǡ񤭹ޤƤ֤...
      long int length2 = 10;
      RTC::RingBuffer<int> buff2(length2);
			
      for (int i = 0; i < length2; i++) {
	// (2) ...ǡisNewȽ꤬ˤʤ뤫
	int writeValue = i + 123;
	buff2.write(writeValue);
	CPPUNIT_ASSERT_EQUAL(false, buff2.empty());
				
	// (3) ...ǡΥǡɽФԤäisNewȽ꤬
	// ˤʤ뤫
	int readValue;
	buff2.read(readValue);
	CPPUNIT_ASSERT_EQUAL(true, buff2.empty());
      }
			
      // Хåե˴ʬǡ񤭹ޤƤ֤...
      long int length3 = 10;
      RTC::RingBuffer<int> buff3(length3);
      for (int i = 0; i < length3 / 2; i++) {
	buff3.write(i + 456);
      }
			
      for (int i = 0; i < length3; i++) {
	// (4) ...ǡisNewȽ꤬ˤʤ뤫
	int writeValue = i + 123;
	buff3.write(writeValue);
	CPPUNIT_ASSERT_EQUAL(false, buff3.empty());
				
	// (5) ...ǡΥǡɽФԤäisNewȽ꤬
	// ˤʤ뤫
	int readValue;
	buff3.read(readValue);
	CPPUNIT_ASSERT_EQUAL(true, buff3.empty());
      }
    }
		
  };
}; // namespace RingBuffer

/*
 * Register test suite
 */
CPPUNIT_TEST_SUITE_REGISTRATION(RingBuffer::RingBufferTests);

#ifdef LOCAL_MAIN
int main(int argc, char* argv[])
{

  FORMAT format = TEXT_OUT;
  int target = 0;
  std::string xsl;
  std::string ns;
  std::string fname;
  std::ofstream ofs;

  int i(1);
  while (i < argc)
    {
      std::string arg(argv[i]);
      std::string next_arg;
      if (i + 1 < argc) next_arg = argv[i + 1];
      else              next_arg = "";

      if (arg == "--text") { format = TEXT_OUT; break; }
      if (arg == "--xml")
	{
	  if (next_arg == "")
	    {
	      fname = argv[0];
	      fname += ".xml";
	    }
	  else
	    {
	      fname = next_arg;
	    }
	  format = XML_OUT;
	  ofs.open(fname.c_str());
	}
      if ( arg == "--compiler"  ) { format = COMPILER_OUT; break; }
      if ( arg == "--cerr"      ) { target = 1; break; }
      if ( arg == "--xsl"       )
	{
	  if (next_arg == "") xsl = "default.xsl"; 
	  else                xsl = next_arg;
	}
      if ( arg == "--namespace" )
	{
	  if (next_arg == "")
	    {
	      std::cerr << "no namespace specified" << std::endl;
	      exit(1); 
	    }
	  else
	    {
	      xsl = next_arg;
	    }
	}
      ++i;
    }
  CppUnit::TextUi::TestRunner runner;
  if ( ns.empty() )
    runner.addTest(CppUnit::TestFactoryRegistry::getRegistry().makeTest());
  else
    runner.addTest(CppUnit::TestFactoryRegistry::getRegistry(ns).makeTest());
  CppUnit::Outputter* outputter = 0;
  std::ostream* stream = target ? &std::cerr : &std::cout;
  switch ( format )
    {
    case TEXT_OUT :
      outputter = new CppUnit::TextOutputter(&runner.result(),*stream);
      break;
    case XML_OUT :
      std::cout << "XML_OUT" << std::endl;
      outputter = new CppUnit::XmlOutputter(&runner.result(),
					    ofs, "shift_jis");
      static_cast<CppUnit::XmlOutputter*>(outputter)->setStyleSheet(xsl);
      break;
    case COMPILER_OUT :
      outputter = new CppUnit::CompilerOutputter(&runner.result(),*stream);
      break;
    }
  runner.setOutputter(outputter);
  runner.run();
  return 0; // runner.run() ? 0 : 1;
}
#endif // MAIN
#endif // RingBuffer_cpp
